@@ -24,7 +24,7 @@ impl BuildValidator for CallValidator {
     fn build(
         schema: &PyDict,
         config: Option<&PyDict>,
-        build_context: &mut BuildContext,
+        build_context: &mut BuildContext<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let py = schema.py();
 
@@ -66,7 +66,7 @@ impl Validator for CallValidator {
 
         let return_value = if let Ok((args, kwargs)) = args.extract::<(&PyTuple, &PyDict)>(py) {
             self.function.call(py, args, Some(kwargs))?
-        } else if let Ok(kwargs) = args.cast_as::<PyDict>(py) {
+        } else if let Ok(kwargs) = args.downcast::<PyDict>(py) {
             self.function.call(py, (), Some(kwargs))?
         } else {
             let msg = "Arguments validator should return a tuple of (args, kwargs) or a dict of kwargs";
