@@ -1,3 +1,4 @@
+use pyo3::exceptions::{PyKeyError, PyTypeError};
 use pyo3::intern;
 use pyo3::prelude::*;
 use pyo3::types::{PyDict, PyList, PyString, PyTuple, PyType};
@@ -12,7 +13,7 @@ use crate::recursion_guard::RecursionGuard;
 use crate::validators::function::convert_err;
 
 use super::arguments::{json_get, json_slice, py_get, py_slice};
-use super::model::create_class;
+use super::model::{create_class, force_setattr};
 use super::with_default::get_default;
 use super::{build_validator, BuildContext, BuildValidator, CombinedValidator, Extra, Validator};
 
@@ -122,6 +123,10 @@ impl Validator for DataclassArgsValidator {
         slots: &'data [CombinedValidator],
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
+        if let Some(field) = extra.assignee_field {
+            // we're validating assignment, completely different logic
+            return self.validate_assignment(py, field, input, extra, slots, recursion_guard);
+        }
         let args = input.validate_dataclass_args(&self.dataclass_name)?;
 
         let output_dict = PyDict::new(py);
@@ -290,6 +295,69 @@ impl Validator for DataclassArgsValidator {
     }
 }
 
+impl DataclassArgsValidator {
+    fn validate_assignment<'s, 'data>(
+        &'s self,
+        py: Python<'data>,
+        field_name: &str,
+        input: &'data impl Input<'data>,
+        extra: &Extra,
+        slots: &'data [CombinedValidator],
+        recursion_guard: &'s mut RecursionGuard,
+    ) -> ValResult<'data, PyObject>
+    where
+        'data: 's,
+    {
+        let dict: &PyDict = match extra.self_instance {
+            Some(d) => d.downcast()?,
+            None => {
+                return Err(
+                    PyTypeError::new_err("self_instance should not be None on dataclass validate_assignment").into(),
+                )
+            }
+        };
+
+        if let Some(field) = self.fields.iter().find(|f| f.name == field_name) {
+            // by using dict but removing the field in question, we match V1 behaviour
+            let data_dict = dict.copy()?;
+            if let Err(err) = data_dict.del_item(field_name) {
+                // KeyError is fine here as the field might not be in the dict
+                if !err.get_type(py).is(PyType::new::<PyKeyError>(py)) {
+                    return Err(err.into());
+                }
+            }
+            let next_extra = Extra {
+                field_name: Some(field_name),
+                assignee_field: None,
+                data: Some(data_dict),
+                ..*extra
+            };
+            match field.validator.validate(py, input, &next_extra, slots, recursion_guard) {
+                Ok(output) => {
+                    dict.set_item(field_name, output)?;
+                    Ok(dict.to_object(py))
+                }
+                Err(ValError::LineErrors(line_errors)) => {
+                    let errors = line_errors
+                        .into_iter()
+                        .map(|e| e.with_outer_location(field_name.into()))
+                        .collect();
+                    Err(ValError::LineErrors(errors))
+                }
+                Err(err) => Err(err),
+            }
+        } else {
+            Err(ValError::new_with_loc(
+                ErrorType::NoSuchAttribute {
+                    attribute: field_name.to_string(),
+                },
+                input,
+                field_name.to_string(),
+            ))
+        }
+    }
+}
+
 #[derive(Debug, Clone)]
 pub struct DataclassValidator {
     strict: bool,
@@ -341,6 +409,15 @@ impl Validator for DataclassValidator {
         slots: &'data [CombinedValidator],
         recursion_guard: &'s mut RecursionGuard,
     ) -> ValResult<'data, PyObject> {
+        if let Some(self_instance) = extra.self_instance {
+            // in the case that self_instance is Some, we're calling validation from within `BaseModel.__init__`
+            // or from `validate_assignment`
+            return if extra.assignee_field.is_some() {
+                self.validate_assignment(py, self_instance, input, extra, slots, recursion_guard)
+            } else {
+                self.validate_init(py, self_instance, input, extra, slots, recursion_guard)
+            };
+        }
         let class = self.class.as_ref(py);
 
         // we only do the is_exact_instance in strict mode
@@ -357,20 +434,9 @@ impl Validator for DataclassValidator {
             ))
         } else {
             let input = input.maybe_subclass_dict(class)?;
-            let output = self.validator.validate(py, input, extra, slots, recursion_guard)?;
-            let (dc_dict, post_init_kwargs): (&PyAny, &PyAny) = output.extract(py)?;
-            let dc = create_class(self.class.as_ref(py), dc_dict, None)?;
-
-            if let Some(ref post_init) = self.post_init {
-                let post_init = post_init.as_ref(py);
-                let r = if post_init_kwargs.is_none() {
-                    dc.call_method0(py, post_init)
-                } else {
-                    let args = post_init_kwargs.downcast::<PyTuple>()?;
-                    dc.call_method1(py, post_init, args)
-                };
-                r.map_err(|e| convert_err(py, e, input))?;
-            }
+            let val_output = self.validator.validate(py, input, extra, slots, recursion_guard)?;
+            let dc = create_class(self.class.as_ref(py))?;
+            self.set_dict_call(py, dc.as_ref(py), val_output, input)?;
             Ok(dc)
         }
     }
@@ -379,3 +445,69 @@ impl Validator for DataclassValidator {
         &self.name
     }
 }
+
+impl DataclassValidator {
+    /// here we just call the inner validator, then set attributes on `self_instance`
+    fn validate_init<'s, 'data>(
+        &'s self,
+        py: Python<'data>,
+        self_instance: &'s PyAny,
+        input: &'data impl Input<'data>,
+        extra: &Extra,
+        slots: &'data [CombinedValidator],
+        recursion_guard: &'s mut RecursionGuard,
+    ) -> ValResult<'data, PyObject> {
+        // we need to set `self_instance` to None for nested validators as we don't want to operate on the self_instance
+        // instance anymore
+        let new_extra = Extra {
+            self_instance: None,
+            ..*extra
+        };
+        let val_output = self.validator.validate(py, input, &new_extra, slots, recursion_guard)?;
+
+        self.set_dict_call(py, self_instance, val_output, input)?;
+
+        Ok(self_instance.into_py(py))
+    }
+
+    #[allow(clippy::too_many_arguments)]
+    fn validate_assignment<'s, 'data>(
+        &'s self,
+        py: Python<'data>,
+        self_instance: &'s PyAny,
+        input: &'data impl Input<'data>,
+        extra: &Extra,
+        slots: &'data [CombinedValidator],
+        recursion_guard: &'s mut RecursionGuard,
+    ) -> ValResult<'data, PyObject> {
+        // inner validator takes care of updating dict, here we just need to update fields_set
+        let next_extra = Extra {
+            self_instance: self_instance.get_attr(intern!(py, "__dict__")),
+            ..*extra
+        };
+        self.validator.validate(py, input, &next_extra, slots, recursion_guard)
+    }
+
+    fn set_dict_call<'s, 'data>(
+        &'s self,
+        py: Python<'data>,
+        dc: &PyAny,
+        val_output: PyObject,
+        input: &'data impl Input<'data>,
+    ) -> ValResult<'data, ()> {
+        let (dc_dict, post_init_kwargs): (&PyAny, &PyAny) = val_output.extract(py)?;
+        force_setattr(py, dc, intern!(py, "__dict__"), dc_dict)?;
+
+        if let Some(ref post_init) = self.post_init {
+            let post_init = post_init.as_ref(py);
+            let r = if post_init_kwargs.is_none() {
+                dc.call_method0(post_init)
+            } else {
+                let args = post_init_kwargs.downcast::<PyTuple>()?;
+                dc.call_method1(post_init, args)
+            };
+            r.map_err(|e| convert_err(py, e, input))?;
+        }
+        Ok(())
+    }
+}
