@@ -584,3 +584,180 @@ def test_dataclass_field_wrap_validator2():
     v = SchemaValidator(schema)
     foo = v.validate_python({'a': 1, 'b': b'hello'})
     assert dataclasses.asdict(foo) == {'a': 1, 'b': 'hello world!'}
+
+
+def test_dataclass_self_init():
+    @dataclasses.dataclass(init=False)
+    class Foo:
+        a: str
+        b: bool
+
+        def __init__(self, *args, **kwargs):
+            v.validate_python(ArgsKwargs(args, kwargs), self_instance=self)
+
+    schema = core_schema.dataclass_schema(
+        Foo,
+        core_schema.dataclass_args_schema(
+            'Foo',
+            [
+                core_schema.dataclass_field(name='a', schema=core_schema.str_schema(), kw_only=False),
+                core_schema.dataclass_field(name='b', schema=core_schema.bool_schema(), kw_only=False),
+            ],
+        ),
+    )
+    v = SchemaValidator(schema)
+
+    foo = Foo(b'hello', 'True')
+    assert dataclasses.is_dataclass(foo)
+    assert dataclasses.asdict(foo) == {'a': 'hello', 'b': True}
+
+
+def test_dataclass_self_init_alias():
+    @dataclasses.dataclass(init=False)
+    class Foo:
+        a: str
+        b: bool
+
+    schema = core_schema.dataclass_schema(
+        Foo,
+        core_schema.dataclass_args_schema(
+            'Foo',
+            [
+                core_schema.dataclass_field(name='a', schema=core_schema.str_schema(), validation_alias='aAlias'),
+                core_schema.dataclass_field(name='b', schema=core_schema.bool_schema(), validation_alias='bAlias'),
+            ],
+        ),
+    )
+    v = SchemaValidator(schema)
+
+    def __init__(self, *args, **kwargs):
+        v.validate_python(ArgsKwargs(args, kwargs), self_instance=self)
+
+    Foo.__init__ = __init__
+
+    foo = Foo(aAlias=b'hello', bAlias='True')
+    assert dataclasses.is_dataclass(foo)
+    assert dataclasses.asdict(foo) == {'a': 'hello', 'b': True}
+
+
+def test_dataclass_self_init_post_init():
+    calls = []
+
+    @dataclasses.dataclass(init=False)
+    class Foo:
+        a: str
+        b: bool
+        # _: dataclasses.KW_ONLY
+        c: dataclasses.InitVar[int]
+
+        def __init__(self, *args, **kwargs):
+            v.validate_python(ArgsKwargs(args, kwargs), self_instance=self)
+
+        def __post_init__(self, c):
+            calls.append(c)
+
+    schema = core_schema.dataclass_schema(
+        Foo,
+        core_schema.dataclass_args_schema(
+            'Foo',
+            [
+                core_schema.dataclass_field(name='a', schema=core_schema.str_schema(), kw_only=False),
+                core_schema.dataclass_field(name='b', schema=core_schema.bool_schema(), kw_only=False),
+                core_schema.dataclass_field(name='c', schema=core_schema.int_schema(), init_only=True),
+            ],
+            collect_init_only=True,
+        ),
+        post_init=True,
+    )
+    v = SchemaValidator(schema)
+
+    foo = Foo(b'hello', 'True', c='123')
+    assert dataclasses.is_dataclass(foo)
+    assert dataclasses.asdict(foo) == {'a': 'hello', 'b': True}
+    assert calls == [123]
+
+
+def test_dataclass_validate_assignment():
+    schema = core_schema.dataclass_schema(
+        FooDataclass,
+        core_schema.dataclass_args_schema(
+            'FooDataclass',
+            [
+                core_schema.dataclass_field(name='a', schema=core_schema.str_schema(), kw_only=False),
+                core_schema.dataclass_field(name='b', schema=core_schema.bool_schema(), kw_only=False),
+            ],
+        ),
+    )
+    v = SchemaValidator(schema)
+
+    foo = v.validate_python({'a': 'hello', 'b': 'True'})
+    assert dataclasses.asdict(foo) == {'a': 'hello', 'b': True}
+    v.validate_assignment(foo, 'a', b'world')
+    assert dataclasses.asdict(foo) == {'a': 'world', 'b': True}
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(foo, 'a', 123)
+    # insert_assert(exc_info.value.errors())
+    assert exc_info.value.errors() == [
+        {'type': 'string_type', 'loc': ('a',), 'msg': 'Input should be a valid string', 'input': 123}
+    ]
+
+    with pytest.raises(ValidationError) as exc_info:
+        v.validate_assignment(foo, 'c', 123)
+    assert exc_info.value.errors() == [
+        {
+            'type': 'no_such_attribute',
+            'loc': ('c',),
+            'msg': "Object has no attribute 'c'",
+            'input': 123,
+            'ctx': {'attribute': 'c'},
+        }
+    ]
+
+    # wrong arguments
+    with pytest.raises(TypeError, match='self_instance should not be None on dataclass validate_assignment'):
+        v.validate_assignment('field_a', 'c', 123)
+
+
+def test_validate_assignment_function():
+    @dataclasses.dataclass
+    class MyDataclass:
+        field_a: str
+        field_b: int
+        field_c: int
+
+    calls = []
+
+    def func(x, info):
+        calls.append(str(info))
+        return x * 2
+
+    v = SchemaValidator(
+        core_schema.dataclass_schema(
+            MyDataclass,
+            core_schema.dataclass_args_schema(
+                'MyDataclass',
+                [
+                    core_schema.dataclass_field('field_a', core_schema.str_schema()),
+                    core_schema.dataclass_field(
+                        'field_b', core_schema.field_after_validation_function(func, core_schema.int_schema())
+                    ),
+                    core_schema.dataclass_field('field_c', core_schema.int_schema()),
+                ],
+            ),
+        )
+    )
+
+    m = v.validate_python({'field_a': 'x', 'field_b': 123, 'field_c': 456})
+    assert m.field_a == 'x'
+    assert m.field_b == 246
+    assert m.field_c == 456
+    assert calls == ["ValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')"]
+
+    v.validate_assignment(m, 'field_b', '111')
+
+    assert m.field_b == 222
+    assert calls == [
+        "ValidationInfo(config=None, context=None, data={'field_a': 'x'}, field_name='field_b')",
+        "ValidationInfo(config=None, context=None, data={'field_a': 'x', 'field_c': 456}, field_name='field_b')",
+    ]
