@@ -242,7 +242,7 @@ def test_empty_choices():
 
 def test_one_choice():
     v = SchemaValidator({'type': 'union', 'choices': [{'type': 'str'}]})
-    assert plain_repr(v) == 'SchemaValidator(name="str",validator=Str(StrValidator{strict:false}),slots=[])'
+    assert plain_repr(v) == 'SchemaValidator(title="str",validator=Str(StrValidator{strict:false}),slots=[])'
     assert v.validate_python('hello') == 'hello'
 
 
